from helper import ExtendedEnum


class AddBenefitValidationErrorCode(ExtendedEnum):
    NDISCOUNT_ERROR_USER_MIN_LEVEL = "The user applying the benefit does not have the level required by the benefit"
