from typing import Optional


class Account(object):
    def __init__(
        self,
        account_type,    # type: int
        amount,          # type: Optional[float]
        tender_id,       # type: int
        operation_date,  # type: str
        glaccount,       # type: str
        is_undeclared,   # type: bool
    ):
        # type: (...) -> None

        self.account_type = account_type
        self.amount = amount
        self.tender_id = tender_id
        self.operation_date = operation_date
        self.glaccount = glaccount
        self.is_undeclared = is_undeclared
