class ConfirmFinishPaymentData(object):
    def __init__(
        self,
        pos_id,     # type: str
        status      # type: str
    ):
        # type: (...) -> None

        self.pos_id = int(pos_id)
        self.status = int(status)

    def __repr__(self):
        return """ConfirmFinishPaymentData(pos_id={}, status={})""".format(
            self.pos_id,
            self.status
        )

    def __str__(self):
        return self.__repr__()
