# encoding: utf-8
from typing import Any


class CardBrand(object):
    def __init__(
        self,
        row_database    # type: Any
    ):
        # type: (...) -> None

        self.brand = row_database.get_entry("Bandeira")
        self.description = row_database.get_entry("Descricao")
