from typing import Dict


class Exclusions(object):
    def __init__(
        self,
        obj,  # type: Dict
    ):
        # type: (...) -> None

        self.items = list(obj.get("items")) if obj.get("items") else None
        self.groups = list(obj.get("groups")) if obj.get("groups") else None
