from pos_model import TenderType


def is_electronic_tender_id(
    tender_id,      # type: int
):
    electronic_tender_ids = [
        TenderType.credit,
        TenderType.debit,
        TenderType.voucher,
        TenderType.digital_wallet,
        TenderType.gift_card,
    ]
    return int(tender_id) in electronic_tender_ids
