from messagebus import TokenCreator, TokenPriority


class BusTokens(object):
    TK_AUDITLOG_RESYNC = TokenCreator.create_token(TokenPriority.high, "14", "1")
    TK_SALE_ORDER_MODIFIED = TokenCreator.create_token(TokenPriority.low, "A", "32")
