class FiscalStatusDto(object):
    def __init__(
            self,
            status,     # type: bool
            message,    # type: str
    ):
        # type: (...) -> None

        self.status = status
        self.message = message
