from typing import List

from ._RequesterType import RequesterType


class DeviceConfig(object):
    def __init__(
        self,
        requester_type,    # type: RequesterType
        requester_id,      # type: int
        default_device,    # type: str
        fallback_devices,  # type: List[str]
    ):
        # type: (...) -> None

        self.requester_type = requester_type
        self.requester_id = requester_id
        self.default_device = default_device
        self.fallback_devices = fallback_devices
