from helper import ExtendedEnum


class ApplyCouponErrorCode(ExtendedEnum):
    COUPON_NOT_FOUND = "The voucher id was not found"
    CUSTOM_ERROR = "The voucher is not available based on custom partner rules"
    COUPON_LOCKED = "The voucher is being used in another pos"
    COUPON_ALREADY_USED = "The voucher was already used"
    NDISCOUNT_ERROR = "The voucher could no be used because the conditions where not validated by the nDiscount"
