from multiprocessing import Condition


class MobileServiceFinder(object):
    def __init__(self):
        self.available_sitef_lock = Condition()

    def lock_sitef(self):
        self.available_sitef_lock.acquire()

    def release_sitef(self):
        self.available_sitef_lock.release()
