from ._CommonProcessor import _CommonProcessor


class CancelPaymentProcessor(_CommonProcessor):
    def parse_data(self, data, event_type):
        return None

    def call_business(self, _):
        processor = self.payment_router.get_payment_processor()
        if processor:
            return processor.cancel_payment()
