from typing import Optional


class ApplyContext(object):
    def __init__(
        self,
        pos_id,             # type: int
        originator_pos_id,  # type: int
        user_auth_level,    # type: int
        applier,            # type: Optional[str]
    ):
        self.pos_id = pos_id
        self.originator_pos_id = originator_pos_id
        self.user_auth_level = user_auth_level
        self.applier = applier
