from modelcommon.printer import SeparatorTag

import constants
from report.command import CutPaperCommand

from .._CommandProcessor import CommandProcessor


class CutPaperCommandProcessor(CommandProcessor):
    def get_bytes(self, command):
        if not isinstance(command, CutPaperCommand):
            return ""

        return b'{0}SpaceCutByTable{1}'.format(SeparatorTag.start, SeparatorTag.end)
